@@ -1,18 +1,16 @@
 /* Copyright(C) 2007- Tasuku SUENAGA
 
-  This library is free software; you can redistribute it and/or
-  modify it under the terms of the GNU Lesser General Public
-  License as published by the Free Software Foundation; either
-  version 2.1 of the License, or (at your option) any later version.
-
-  This library is distributed in the hope that it will be useful,
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  Lesser General Public License for more details.
-
-  You should have received a copy of the GNU Lesser General Public
-  License along with this library; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
+CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
+EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
+PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
+PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
+LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 #include <Python.h>
 #include <tcbdb.h>
@@ -237,7 +235,8 @@ typedef struct {
     static char *kwlist[] = {"key", "value", NULL}; \
   \
     if (!PyArg_ParseTupleAndKeywords(args, keywds, "s#s#:" #method, kwlist, \
-                                     &key, &key_len, &value, &value_len)) { \
+                                     &key, &key_len, \
+                                     &value, &value_len)) { \
       return NULL; \
     } \
     Py_BEGIN_ALLOW_THREADS \
@@ -324,7 +323,7 @@ typedef struct {
       PyErr_SetString(PyExc_TypeError, "only string is allowed in []"); \
       return NULL; \
     } \
-    key = PyString_AS_STRING(_key); \
+    key = PyString_AsString(_key); \
     key_len = PyString_GET_SIZE(_key); \
     if (!key || !key_len) { \
       return NULL; \
@@ -346,7 +345,7 @@ typedef struct {
   int \
   func(type *self, PyObject *_key) { \
     bool result; \
-    char *key = PyString_AS_STRING(_key); \
+    char *key = PyString_AsString(_key); \
     int key_len = PyString_GET_SIZE(_key); \
   \
     if (!key || !key_len) { \
@@ -367,7 +366,7 @@ typedef struct {
   int \
   func(type *self, PyObject *_key, PyObject *_value) { \
     bool result; \
-    char *key = PyString_AS_STRING(_key), *value = PyString_AS_STRING(_value); \
+    char *key = PyString_AsString(_key), *value = PyString_AsString(_value); \
     int key_len = PyString_GET_SIZE(_key), value_len = PyString_GET_SIZE(_value); \
   \
     if (!key || !key_len || !value) { \
@@ -398,7 +397,7 @@ typedef struct {
 #define TCXDB_Contains(func,type,call,member) \
   static int \
   func(type *self, PyObject *_key) { \
-    char *key = PyString_AS_STRING(_key); \
+    char *key = PyString_AsString(_key); \
     int key_len = PyString_GET_SIZE(_key), value_len; \
   \
     if (!key || !key_len) { \
@@ -421,7 +420,7 @@ typedef struct {
   static PyObject * \
   func(type *self, PyObject *_key) { \
     PyObject *ret; \
-    char *key = PyString_AS_STRING(_key), *value; \
+    char *key = PyString_AsString(_key), *value; \
     int key_len = PyString_GET_SIZE(_key), value_len; \
   \
     if (!key || !key_len) { \
@@ -476,6 +475,27 @@ typedef struct {
   if (value) { tcxstrdel(value); } \
   return ret;
 
+#define TCLIST2PyList() \
+  if (!list) { \
+    raise_tcbdb_error(self->bdb); \
+    return NULL; \
+  } else { \
+    PyObject *ret; \
+    int i, n = tclistnum(list); \
+    if ((ret = PyList_New(n))) { \
+      for (i = 0; i < n; i++) { \
+        int value_len; \
+        PyObject *_value; \
+        const char *value; \
+        value = tclistval(list, i, &value_len); \
+        _value = PyString_FromStringAndSize(value, value_len); \
+        PyList_SET_ITEM(ret, i, _value); \
+      } \
+    } \
+    tclistdel(list); \
+    return ret; \
+  }
+
 /*** TCHDB ***/
 
 #define PyTCHDB_TUNE(a,b,c) \
@@ -1399,25 +1419,7 @@ PyTCBDB_getlist(PyTCBDB *self, PyObject *args, PyObject *keywds) {
   list = tcbdbget4(self->bdb, key, key_len);
   Py_END_ALLOW_THREADS
 
-  if (!list) {
-    raise_tcbdb_error(self->bdb);
-    return NULL;
-  } else {
-    PyObject *ret;
-    int i, n = tclistnum(list);
-    if ((ret = PyList_New(n))) {
-      for (i = 0; i < n; i++) {
-        int value_len;
-        PyObject *_value;
-        const char *value;
-        value = tclistval(list, i, &value_len);
-        _value = PyString_FromStringAndSize(value, value_len);
-        PyList_SET_ITEM(ret, i, _value);
-      }
-    }
-    tclistdel(list);
-    return ret;
-  }
+  TCLIST2PyList()
 }
 
 INT_KEYARGS(PyTCBDB_vnum,PyTCBDB,vnum,tcbdbvnum,bdb,raise_tcbdb_error)
@@ -1433,6 +1435,44 @@ STRING_NOARGS(PyTCBDB_path,PyTCBDB,tcbdbpath,bdb,raise_tcbdb_error)
 PY_U_LONG_LONG_NOARGS(PyTCBDB_rnum, PyTCBDB, tcbdbrnum, bdb, tcbdbecode, raise_tcbdb_error)
 PY_U_LONG_LONG_NOARGS(PyTCBDB_fsiz, PyTCBDB, tcbdbrnum, bdb, tcbdbecode, raise_tcbdb_error)
 
+static PyObject *
+PyTCBDB_range(PyTCBDB *self, PyObject *args, PyObject *keywds) {
+  TCLIST *list;
+  char *bkey, *ekey;
+  int bkey_len, binc, ekey_len, einc, max;
+  static char *kwlist[] = {"bkey", "binc", "ekey", "einc", "max", NULL};
+
+  if (!PyArg_ParseTupleAndKeywords(args, keywds, "z#iz#ii:range", kwlist,
+                                   &bkey, &bkey_len, &binc,
+                                   &ekey, &ekey_len, &einc, &max)) {
+    return NULL;
+  }
+  Py_BEGIN_ALLOW_THREADS
+  list = tcbdbrange(self->bdb, bkey, bkey_len, binc,
+                               ekey, ekey_len, einc, max);
+  Py_END_ALLOW_THREADS
+
+  TCLIST2PyList()
+}
+
+static PyObject *
+PyTCBDB_rangefwm(PyTCBDB *self, PyObject *args, PyObject *keywds) {
+  int max;
+  TCLIST *list;
+  char *prefix;
+  static char *kwlist[] = {"prefix", "max", NULL};
+
+  if (!PyArg_ParseTupleAndKeywords(args, keywds, "si:rangefwm", kwlist,
+                                   &prefix, &max)) {
+    return NULL;
+  }
+  Py_BEGIN_ALLOW_THREADS
+  list = tcbdbrange3(self->bdb, prefix, max);
+  Py_END_ALLOW_THREADS
+
+  TCLIST2PyList()
+}
+
 /* TODO: features for experts */
 
 TCXDB_Contains(PyTCBDB_Contains,PyTCBDB,tcbdbvsiz,bdb)
@@ -1725,6 +1765,12 @@ static PyMethodDef PyTCBDB_methods[] = {
   {"curnew", (PyCFunction)PyTCBDB_curnew,
    METH_NOARGS,
    "Create a cursor object."},
+  {"range", (PyCFunction)PyTCBDB_range,
+   METH_VARARGS | METH_KEYWORDS,
+   ""},
+  {"rangefwm", (PyCFunction)PyTCBDB_rangefwm,
+   METH_VARARGS | METH_KEYWORDS,
+   ""},
   {"__contains__", (PyCFunction)PyTCBDB___contains__,
    METH_O | METH_COEXIST,
    ""},
